@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.2 1999/12/03 12:58:29 gisle Exp $
+/* $Id: hparser.c,v 2.5 1999/12/04 13:10:52 gisle Exp $
  *
  * Copyright 1999, Gisle Aas.
  *
@@ -13,6 +13,21 @@
 #include "hctype.h"    /* isH...() macros */
 #include "tokenpos.h"  /* dTOKEN; PUSH_TOKEN() */
 
+
+static
+struct literal_tag {
+  int len;
+  char* str;
+}
+literal_mode_elem[] =
+{
+  {6, "script"},
+  {5, "style"},
+  {3, "xmp"},
+  {9, "plaintext"},
+  {0, 0}
+};
+
 /*
  * Parser functions.
  *
@@ -32,11 +47,12 @@
  *                      has recongnized something.
  */
 
-EXTERN void
+static void
 report_event(PSTATE* p_state,
 	     event_id_t event,
 	     char *beg, char *end,
 	     token_pos_t *tokens, int num_tokens,
+	     STRLEN offset,
 	     SV* self
 	    )
 {
@@ -221,6 +237,10 @@ report_event(PSTATE* p_state,
       }
       break;
 
+    case '=':
+      arg = sv_2mortal(newSViv(p_state->chunk_offset + offset));
+      break;
+
     case 'E':
       /* event */
       assert(event >= 0 && event < EVENT_COUNT);
@@ -274,7 +294,7 @@ report_event(PSTATE* p_state,
 }
 
 
-EXTERN SV*
+static SV*
 attrspec_compile(SV* src)
 {
   SV* attrspec = newSVpvn("", 0);
@@ -297,6 +317,7 @@ attrspec_compile(SV* src)
     hv_store(names, "dtext", 5,         newSVpvn("D", 1), 0);
     hv_store(names, "cdata_flag", 10,   newSVpvn("c", 1), 0);
     hv_store(names, "event", 5,         newSVpvn("E", 1), 0);
+    hv_store(names, "offset", 6,        newSVpvn("=", 1), 0);
   }
 
   while (isHSPACE(*s))
@@ -356,8 +377,8 @@ attrspec_compile(SV* src)
 }
 
 
-EXTERN char*
-parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_comment(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
   char *s = beg;
 
@@ -386,7 +407,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 	report_event(p_state, E_COMMENT,
 		    beg - 4, s,
 		    tokens, num_tokens,
-		    self);
+		    offset, self);
 	FREE_TOKENS;
 
 	return s;
@@ -430,7 +451,8 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 	if (*s == '>') {
 	  s++;
 	  /* yup */
-	  report_event(p_state, E_COMMENT, beg-4, s, &token_pos, 1, self);
+	  report_event(p_state, E_COMMENT, beg-4, s, &token_pos, 1,
+		       offset, self);
 	  return s;
 	}
       }
@@ -450,7 +472,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 
 #ifdef MARKED_SECTION
 
-EXTERN void
+static void
 marked_section_update(PSTATE* p_state)
 {
   /* we look at p_state->ms_stack to determine p_state->ms */
@@ -496,7 +518,7 @@ marked_section_update(PSTATE* p_state)
 }
 
 
-EXTERN char*
+static char*
 parse_marked_section(PSTATE* p_state, char *beg, char *end, SV* self)
 {
   char *s = beg;
@@ -574,8 +596,8 @@ parse_marked_section(PSTATE* p_state, char *beg, char *end, SV* self)
 #endif
 
 
-EXTERN char*
-parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
   char *s = beg + 2;
 
@@ -593,7 +615,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
     /* yes, two dashes seen */
     s++;
 
-    tmp = parse_comment(p_state, s, end, self);
+    tmp = parse_comment(p_state, s, end, offset, self);
     return (tmp == s) ? beg : tmp;
   }
 
@@ -613,7 +635,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
     empty.beg = s;
     empty.end = s;
     s++;
-    report_event(p_state, E_COMMENT, beg, s, &empty, 1, self);
+    report_event(p_state, E_COMMENT, beg, s, &empty, 1, offset, self);
     return s;
   }
 
@@ -689,7 +711,8 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
       goto PREMATURE;
     if (*s == '>') {
       s++;
-      report_event(p_state, E_DECLARATION, beg, s, tokens, num_tokens, self);
+      report_event(p_state, E_DECLARATION, beg, s, tokens, num_tokens,
+		   offset, self);
       FREE_TOKENS;
       return s;
     }
@@ -707,8 +730,8 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 }
 
 
-EXTERN char*
-parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
   char *s = beg;
   SV* attr;
@@ -812,9 +835,10 @@ parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
   if (*s == '>') {
     s++;
     /* done */
-    report_event(p_state, E_START, beg, s, tokens, num_tokens, self);
+    report_event(p_state, E_START, beg, s, tokens, num_tokens, offset, self);
     if (empty_tag)
-      report_event(p_state, E_END, s, s, tokens, 1, self);
+      report_event(p_state, E_END, s, s, tokens, 1,
+		   offset + (s - beg), self);
     FREE_TOKENS;
 
     if (1) {
@@ -858,8 +882,8 @@ parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
 }
 
 
-EXTERN char*
-parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_end(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
   char *s = beg+2;
   hctype_t name_first, name_char;
@@ -885,7 +909,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
       if (*s == '>') {
 	s++;
 	/* a complete end tag has been recognized */
-	report_event(p_state, E_END, beg, s, &tagname, 1, self);
+	report_event(p_state, E_END, beg, s, &tagname, 1, offset, self);
 	return s;
       }
     }
@@ -897,8 +921,9 @@ parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
 }
 
 
-EXTERN char*
-parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_process(PSTATE* p_state, char *beg, char *end,
+	      STRLEN offset, SV* self)
 {
   char *s = beg + 2;
   /* processing instruction */
@@ -920,7 +945,7 @@ parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
     }
 
     /* a complete processing instruction seen */
-    report_event(p_state, E_PROCESS, beg, s, &token_pos, 1, self);
+    report_event(p_state, E_PROCESS, beg, s, &token_pos, 1, offset, self);
     return s;
   }
   else {
@@ -930,8 +955,8 @@ parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
 }
 
 
-EXTERN char*
-parse_null(PSTATE* p_state, char *beg, char *end, SV* self)
+static char*
+parse_null(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
   return 0;
 }
@@ -945,7 +970,7 @@ parse(PSTATE* p_state,
 	   SV* chunk,
 	   SV* self)
 {
-  char *s, *t, *end, *new_pos;
+  char *s, *t, *beg, *end, *new_pos;
   STRLEN len;
 
   if (!chunk || !SvOK(chunk)) {
@@ -955,7 +980,8 @@ parse(PSTATE* p_state,
       STRLEN len;
       char *s = SvPV(p_state->buf, len);
       assert(len);
-      report_event(p_state, E_TEXT, s, s+len, 0, 0, self);
+      report_event(p_state, E_TEXT, s, s+len, 0, 0, 0, self);
+      p_state->chunk_offset += len;
       SvREFCNT_dec(p_state->buf);
       p_state->buf = 0;
     }
@@ -964,16 +990,17 @@ parse(PSTATE* p_state,
 
   if (p_state->buf && SvOK(p_state->buf)) {
     sv_catsv(p_state->buf, chunk);
-    s = SvPV(p_state->buf, len);
+    beg = SvPV(p_state->buf, len);
   }
   else {
-    s = SvPV(chunk, len);
+    beg = SvPV(chunk, len);
   }
 
   if (!len)
     return; /* nothing to do */
 
-  t = s;
+  s = beg;
+  t = beg;
   end = s + len;
 
   while (1) {
@@ -1017,8 +1044,10 @@ parse(PSTATE* p_state,
 	  if (*s == '>') {
 	    s++;
 	    if (t != end_text)
-	      report_event(p_state, E_TEXT, t, end_text, 0, 0, self);
-	    report_event(p_state, E_END,  end_text, s, &end_token, 1, self);
+	      report_event(p_state, E_TEXT, t, end_text, 0, 0,
+			   t - beg, self);
+	    report_event(p_state, E_END,  end_text, s, &end_token, 1,
+			 end_text - beg, self);
 	    p_state->literal_mode = 0;
 	    t = s;
 	  }
@@ -1039,7 +1068,7 @@ parse(PSTATE* p_state,
 	    s++;
 	    /* marked section end */
 	    if (t != end_text)
-	      report_event(p_state, E_TEXT, t, end_text, 0, 0, self);
+	      report_event(p_state, E_TEXT, t, end_text, 0, 0, t - beg, self);
 	    t = s;
 	    SvREFCNT_dec(av_pop(p_state->ms_stack));
 	    marked_section_update(p_state);
@@ -1064,7 +1093,8 @@ parse(PSTATE* p_state,
 	  s++;
 	  if (*s == '>') {
 	    s++;
-	    report_event(p_state, E_TEXT, t, end_text, 0, 0, self);
+	    report_event(p_state, E_TEXT, t, end_text, 0, 0,
+			 t - beg, self);
 	    SvREFCNT_dec(av_pop(p_state->ms_stack));
 	    marked_section_update(p_state);    
 	    t = s;
@@ -1077,7 +1107,7 @@ parse(PSTATE* p_state,
     }
     if (s != t) {
       if (*s == '<') {
-	report_event(p_state, E_TEXT, t, s, 0, 0, self);
+	report_event(p_state, E_TEXT, t, s, 0, 0, t - beg, self);
 	t = s;
       }
       else {
@@ -1096,7 +1126,7 @@ parse(PSTATE* p_state,
 	}
 	s++;
 	if (s != t)
-	  report_event(p_state, E_TEXT, t, s, 0, 0, self);
+	  report_event(p_state, E_TEXT, t, s, 0, 0, beg - t, self);
 	break;
       }
     }
@@ -1107,7 +1137,7 @@ parse(PSTATE* p_state,
     /* next char is known to be '<' and pointed to by 't' as well as 's' */
     s++;
 
-    if ( (new_pos = parsefunc[*s](p_state, t, end, self))) {
+    if ( (new_pos = parsefunc[*s](p_state, t, end, t - beg, self))) {
       if (new_pos == t) {
 	/* no progress, need more data to know what it is */
 	s = t;
@@ -1124,6 +1154,8 @@ parse(PSTATE* p_state,
 
  DONE:
 
+  p_state->chunk_offset += (s - beg);
+
   if (s == end) {
     if (p_state->buf) {
       SvOK_off(p_state->buf);
